@@ -3,25 +3,26 @@
     Copyright (C) 1998  Itai Tzur <itzur@actcom.co.il>
     Copyright (C) 1999  Sean 'Shaleh' Perry <shaleh@debian.org>
     Copyright (C) 2004  Pascal Hakim <pasc@redellipse.net>
- 
+
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
- 
+
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
- 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  
     The GNU General Public License can also be found in the file
     `COPYING' that comes with the Anacron source distribution.
 */
 
+#include "config.h"
 
 #include <time.h>
 #include <sys/time.h>
@@ -36,21 +37,21 @@
 #include <locale.h>
 #include "global.h"
 #include "gregor.h"
+#include "cronie_common.h"
 
 pid_t primary_pid;
 int day_now;
 int year, month, day_of_month;                 /* date anacron started */
 
 char *program_name;
-char *anacrontab;
-char *spooldir;
+char *anacrontab = NULL;
+char *spooldir = NULL;
 int serialize, force, update_only, now,
     no_daemon, quiet, testing_only;            /* command-line options */
-char **args;                       /* vector of "job" command-line arguments */
-int nargs;                                     /* number of these */
+char **job_args;                       	       /* vector of "job" command-line arguments */
+int job_nargs;                                 /* number of these */
 char *defarg = "*";
 int in_background;                             /* are we in the background? */
-int old_umask;                                 /* umask when started */
 sigset_t old_sigmask;                          /* signal mask when started */
 
 job_rec *first_job_rec;
@@ -61,41 +62,39 @@ static volatile int got_sigalrm, got_sigchld, got_sigusr1;
 int running_jobs, running_mailers;              /* , number of */
 int range_start = -1;
 int range_stop = -1;
+int preferred_hour = -1;
 
 static void
-print_version()
+print_version(void)
 {
-    printf("Anacron \n"
+    printf("Anacron from project %s\n"
 	   "Copyright (C) 1998  Itai Tzur <itzur@actcom.co.il>\n"
 	   "Copyright (C) 1999  Sean 'Shaleh' Perry <shaleh@debian.org>\n"
 	   "Copyright (C) 2004  Pascal Hakim <pasc@redellipse.net>\n"
 	   "\n"
 	   "Mail comments, suggestions and bug reports to <pasc@redellipse.net>."
-	   "\n\n");
+	   "\n\n", PACKAGE_STRING);
 }
 
 static void
-print_usage()
+print_usage(void)
 {
-    printf("Usage:  anacron [-s] [-f] [-n] [-d] [-q] [-t anacrontab] [-S spooldir] [job] ...\n"
-	   "        anacron [-S spooldir] -u [job] ...\n"
-	   "        anacron [-V|-h]\n"
-	   "        anacron -T [-t anacrontab]\n"
-	   "\n"
-	   " -s  Serialize execution of jobs\n"
-	   " -f  Force execution of jobs, even before their time\n"
-	   " -n  Run jobs with no delay, implies -s\n"
-	   " -d  Don't fork to the background\n"
-	   " -q  Suppress stderr messages, only applicable with -d\n"
-	   " -u  Update the timestamps without actually running anything\n"
-	   " -t  Use this anacrontab\n"
-	   " -V  Print version information\n"
-	   " -h  Print this message\n"
-	   " -T  Test an anacrontab\n"
-	   " -S  Select a different spool directory\n"
-	   "\n"
-	   "See the manpage for more details.\n"
-	   "\n");
+    printf("Usage:\n");
+    printf(" %s [options] [job] ...\n", program_name);
+    printf(" %s -T [-t anacrontab-file]\n", program_name);
+    printf("\nOptions:\n");
+    printf(" -s         Serialize execution of jobs\n");
+    printf(" -f         Force execution of jobs, even before their time\n");
+    printf(" -n         Run jobs with no delay, implies -s\n");
+    printf(" -d         Don't fork to the background\n");
+    printf(" -q         Suppress stderr messages, only applicable with -d\n");
+    printf(" -u         Update the timestamps without actually running anything\n");
+    printf(" -V         Print version information\n");
+    printf(" -h         Print this message\n");
+    printf(" -t <file>  Use alternative anacrontab\n");
+    printf(" -T         Test an anacrontab\n");
+    printf(" -S <dir>   Select a different spool directory\n");
+    printf("\nSee the anacron(8) manpage for more details.\n");
 }
 
 static void
@@ -129,20 +128,22 @@ parse_opts(int argc, char *argv[])
 	    quiet = 1;
 	    break;
 	case 't':
+	    free(anacrontab);
 	    anacrontab = strdup(optarg);
 	    break;
 	case 'T':
 	    testing_only = 1;
 	    break;
 	case 'S':
+	    free(spooldir);
 	    spooldir = strdup(optarg);
 	    break;
 	case 'V':
 	    print_version();
-	    exit(0);
+	    exit(EXIT_SUCCESS);
 	case 'h':
 	    print_usage();
-	    exit(0);
+	    exit(EXIT_SUCCESS);
 	case '?':
 	    fprintf(stderr, "%s: invalid option: %c\n",
 		    program_name, optopt);
@@ -154,18 +155,18 @@ parse_opts(int argc, char *argv[])
     if (optind == argc)
     {
 	/* no arguments. Equivalent to: `*' */
-	nargs = 1;
-	args = &defarg;
+	job_nargs = 1;
+	job_args = &defarg;
     }
     else
     {
-	nargs = argc - optind;
-	args = argv + optind;
+	job_nargs = argc - optind;
+	job_args = argv + optind;
     }
 }
 
 pid_t
-xfork()
+xfork(void)
 /* Like fork(), only never returns on failure */
 {
     pid_t pid;
@@ -201,24 +202,26 @@ xclose(int fd)
 }
 
 static void
-go_background()
+go_background(void)
 /* Become a daemon. The foreground process exits successfully. */
 {
     pid_t pid;
 
     /* stdin is already closed */
 
-    if (fclose(stdout)) die_e("Can't close stdout");
-    xopen(1, "/dev/null", O_WRONLY);
+    xclose(STDOUT_FILENO);
+    /* coverity[leaked_handle] – fd 1 closed automatically */
+    xopen(STDOUT_FILENO, "/dev/null", O_WRONLY);
 
-    if (fclose(stderr)) die_e("Can't close stderr");
-    xopen(2, "/dev/null", O_WRONLY);
+    xclose(STDERR_FILENO);
+    /* coverity[leaked_handle] – fd 2 closed automatically */
+    xopen(STDERR_FILENO, "/dev/null", O_WRONLY);
 
     pid = xfork();
     if (pid != 0)
     {
 	/* parent */
-	exit(0);
+	exit(EXIT_SUCCESS);
     }
     else
     {
@@ -229,26 +232,26 @@ go_background()
     }
 }
 
-void
-handle_sigalrm()
+static void
+handle_sigalrm(int unused ATTRIBUTE_UNUSED)
 {
     got_sigalrm = 1;
 }
 
-void
-handle_sigchld()
+static void
+handle_sigchld(int unused ATTRIBUTE_UNUSED)
 {
     got_sigchld = 1;
 }
 
-void
-handle_sigusr1()
+static void
+handle_sigusr1(int unused ATTRIBUTE_UNUSED)
 {
     got_sigusr1 = 1;
 }
 
 static void
-set_signal_handling()
+set_signal_handling(void)
 /* We only use SIGALRM, SIGCHLD and SIGUSR1, and we unblock them only
  * in wait_signal().
  */
@@ -285,7 +288,7 @@ set_signal_handling()
 }
 
 static void
-wait_signal()
+wait_signal(void)
 /* Return after a signal is caught */
 {
     sigset_t ss;
@@ -298,7 +301,7 @@ wait_signal()
 }
 
 static void
-wait_children()
+wait_children(void)
 /* Wait until we have no more children (of any kind) */
 {
     while (running_jobs > 0 || running_mailers > 0)
@@ -312,14 +315,14 @@ wait_children()
 }
 
 static void
-orderly_termination()
+orderly_termination(void)
 /* Execution is diverted here, when we get SIGUSR1 */
 {
     explain("Received SIGUSR1");
     got_sigusr1 = 0;
     wait_children();
     explain("Exited");
-    exit(0);
+    exit(EXIT_SUCCESS);
 }
 
 static void
@@ -342,7 +345,7 @@ xsleep(unsigned int n)
 }
 
 static void
-wait_jobs()
+wait_jobs(void)
 /* Wait until there are no running jobs,
  * servicing SIGCHLDs and SIGUSR1s in the meantime.
  */
@@ -357,7 +360,7 @@ wait_jobs()
 }
 
 static void
-record_start_time()
+record_start_time(void)
 {
     struct tm *tm_now;
 
@@ -373,17 +376,17 @@ record_start_time()
 		year, month, day_of_month);
 }
 
-static int
+static unsigned int
 time_till(job_rec *jr)
 /* Return the number of seconds that we have to wait until it's time
  * to start job jr.
  */
 {
-    unsigned int tj, tn;
+    time_t tj, tn;
 
     if (now) return 0;
     tn = time(NULL);
-    tj = start_sec + jr->delay * 60;
+    tj = start_sec + (time_t)jr->delay * 60;
     if (tj < tn) return 0;
     if (tj - tn > 3600*24)
     {
@@ -391,11 +394,11 @@ time_till(job_rec *jr)
 	    jr->ident);
 	return 0;
     }
-    return tj - tn;
+    return (unsigned int)(tj - tn);
 }
 
 static void
-fake_jobs()
+fake_jobs(void)
 {
     int j;
 
@@ -410,7 +413,7 @@ fake_jobs()
 }
 
 static void
-explain_intentions()
+explain_intentions(void)
 {
     int j;
 
@@ -448,7 +451,7 @@ main(int argc, char *argv[])
     if (gettimeofday(&tv, &tz) != 0)
         explain("Can't get exact time, failure.");
 
-    srandom(getpid()+tv.tv_usec);
+    srandom((unsigned int)(getpid() + tv.tv_usec));
 
     if((program_name = strrchr(argv[0], '/')) == NULL)
 	program_name = argv[0];
@@ -471,12 +474,10 @@ main(int argc, char *argv[])
 
     if (chdir(spooldir)) die_e("Can't chdir to %s", spooldir );
 
-    old_umask = umask(0);
-
     if (sigprocmask(0, NULL, &old_sigmask)) die_e("sigset error");
 
-    if (fclose(stdin)) die_e("Can't close stdin");
-    xopen(0, "/dev/null", O_RDONLY);
+    xclose(STDIN_FILENO);
+    xopen(STDIN_FILENO, "/dev/null", O_RDONLY);
 
     if (!no_daemon && !testing_only)
 	go_background();
@@ -490,15 +491,15 @@ main(int argc, char *argv[])
 
     if (testing_only)
     {
-	if (complaints) exit (1);
+	if (complaints) exit (EXIT_FAILURE);
 	
-	exit (0);
+	exit (EXIT_SUCCESS);
     }
 
     if (update_only)
     {
 	fake_jobs();
-	exit(0);
+	exit(EXIT_SUCCESS);
     }
 
     explain_intentions();
@@ -512,5 +513,5 @@ main(int argc, char *argv[])
     }
     wait_children();
     explain("Normal exit (%d job%s run)", njobs, njobs == 1 ? "" : "s");
-    exit(0);
+    exit(EXIT_SUCCESS);
 }
