@@ -2,20 +2,20 @@
     Anacron - run commands periodically
     Copyright (C) 1998  Itai Tzur <itzur@actcom.co.il>
     Copyright (C) 1999  Sean 'Shaleh' Perry <shaleh@debian.org>
- 
+
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
- 
+
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
- 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  
     The GNU General Public License can also be found in the file
     `COPYING' that comes with the Anacron source distribution.
@@ -30,7 +30,7 @@
 #include "matchrx.h"
 
 int
-match_rx(const char *rx, char *string, int n_sub,  /* char **substrings */...)
+match_rx(const char *rx, char *string, unsigned int n_sub,  /* char **substrings */...)
 /* Return 1 if the regular expression "*rx" matches the string "*string",
  * 0 if not, -1 on error.
  * "Extended" regular expressions are used.
@@ -42,11 +42,13 @@ match_rx(const char *rx, char *string, int n_sub,  /* char **substrings */...)
  * This is not the most efficient, or elegant way of doing this.
  */
 {
-	int r, n;
+	int r;
+	unsigned int n;
 	regex_t crx;
 	va_list va;
 	char **substring;
 	regmatch_t *sub_offsets;
+
 	sub_offsets = malloc(sizeof(regmatch_t) * (n_sub + 1));
 	if (sub_offsets == NULL)
 	    return -1;
