@@ -77,7 +77,7 @@ conv2int(const char *s)
     l = strtol(s, NULL, 10);
     /* we use negative as error, so I am really returning unsigned int */
     if (errno == ERANGE || l < 0 || l > INT_MAX) return - 1;
-    return l;
+    return (int)l;
 }
 
 static char *
@@ -94,11 +94,11 @@ Return NULL if no more lines.
 	c = getc(tab);
 	if ((c == '\n' && prev != '\\') || c == EOF)
 	{
-	    if (0 != prev) obstack_1grow(&input_o, prev);
+	    if (0 != prev) obstack_1grow(&input_o, (char)prev);
 	    break;
 	}
 
-	if ('\\' != prev && 0 != prev && '\n' != prev) obstack_1grow(&input_o, prev);
+	if (('\\' != prev || c != '\n') && 0 != prev && '\n' != prev) obstack_1grow(&input_o, (char)prev);
 	else if ('\n' == prev) obstack_1grow(&input_o, ' ');
 
 	prev = c;
@@ -132,10 +132,21 @@ register_env(const char *env_var, const char *value)
     env_rec *er;
     int var_len, val_len;
 
-    var_len = strlen(env_var);
-    val_len = strlen(value);
+    var_len = (int)strlen(env_var);
+    val_len = (int)strlen(value);
+    if (!var_len) {
+        return;
+    }
+
     er = obstack_alloc(&tab_o, sizeof(env_rec));
+    if (er == NULL) {
+        die_e("Cannot allocate memory.");
+    }
+
     er->assign = obstack_alloc(&tab_o, var_len + 1 + val_len + 1);
+    if (er->assign == NULL) {
+        die_e("Cannot allocate memory.");
+    }
     strcpy(er->assign, env_var);
     er->assign[var_len] = '=';
     strcpy(er->assign + var_len + 1, value);
@@ -155,8 +166,8 @@ register_job(const char *periods, const char *delays,
     job_rec *jr;
     int ident_len, command_len;
 
-    ident_len = strlen(ident);
-    command_len = strlen(command);
+    ident_len = (int)strlen(ident);
+    command_len = (int)strlen(command);
     jobs_read++;
     period = conv2int(periods);
     delay = conv2int(delays);
@@ -167,15 +178,24 @@ register_job(const char *periods, const char *delays,
 	return;
     }
     jr = obstack_alloc(&tab_o, sizeof(job_rec));
+    if (jr == NULL) {
+        die_e("Cannot allocate memory.");
+    }
     jr->period = period;
     jr->named_period = 0;
     delay += random_number;
     jr->delay = delay;
     jr->tab_line = line_num;
     jr->ident = obstack_alloc(&tab_o, ident_len + 1);
+    if (jr->ident == NULL) {
+        die_e("Cannot allocate memory.");
+    }
     strcpy(jr->ident, ident);
     jr->arg_num = job_arg_num(ident);
     jr->command = obstack_alloc(&tab_o, command_len + 1);
+    if (jr->command == NULL) {
+        die_e("Cannot allocate memory.");
+    }
     strcpy(jr->command, command);
     jr->job_pid = jr->mailer_pid = 0;
     if (last_job_rec != NULL) last_job_rec->next = jr;
@@ -196,8 +216,8 @@ register_period_job(const char *periods, const char *delays,
     job_rec *jr;
     int ident_len, command_len;
 
-    ident_len = strlen(ident);
-    command_len = strlen(command);
+    ident_len = (int)strlen(ident);
+    command_len = (int)strlen(command);
     jobs_read++;
     delay = conv2int(delays);
     if (delay < 0)
@@ -208,6 +228,9 @@ register_period_job(const char *periods, const char *delays,
     }
 
     jr = obstack_alloc(&tab_o, sizeof(job_rec));
+    if (jr == NULL) {
+        die_e("Cannot allocate memory.");
+    }
     if (!strncmp ("@monthly", periods, 8)) {
 		jr->named_period = 1;
     } else if (!strncmp("@yearly", periods, 7) || !strncmp("@annually", periods, 9) || !strncmp(/* backwards compat misspelling */"@annualy", periods, 8)) {
@@ -225,9 +248,15 @@ register_period_job(const char *periods, const char *delays,
     jr->delay = delay;
     jr->tab_line = line_num;
     jr->ident = obstack_alloc(&tab_o, ident_len + 1);
+    if (jr->ident == NULL) {
+        die_e("Cannot allocate memory.");
+    }
     strcpy(jr->ident, ident);
     jr->arg_num = job_arg_num(ident);
     jr->command = obstack_alloc(&tab_o, command_len + 1);
+    if (jr->command == NULL) {
+        die_e("Cannot allocate memory.");
+    }
     strcpy(jr->command, command);
     jr->job_pid = jr->mailer_pid = 0;
     if (last_job_rec != NULL) last_job_rec->next = jr;
@@ -362,7 +391,7 @@ read_tab(int cwd)
     jobs_read = 0;
     line_num = 0;
     /* Open the anacrontab file */
-    fchdir (cwd);
+    if (fchdir(cwd)) die_e("Can't chdir to original cwd");
     tab = fopen(anacrontab, "r");
     if (chdir(spooldir)) die_e("Can't chdir to %s", spooldir);
 
@@ -417,6 +446,6 @@ arrange_jobs(void)
     job_array = obstack_finish(&tab_o);
 
     /* sort the jobs */
-    qsort(job_array, njobs, sizeof(*job_array),
+    qsort(job_array, (size_t)njobs, sizeof(*job_array),
 	  (int (*)(const void *, const void *))execution_order);
 }
