@@ -23,8 +23,10 @@
 
 #include <ctype.h>
 #include <errno.h>
+#include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
 #include <unistd.h>
 
 #include "globals.h"
@@ -51,7 +53,8 @@ void env_free(char **envp) {
 }
 
 char **env_copy(char **envp) {
-	int count, i, save_errno;
+	int save_errno;
+	size_t count, i;
 	char **p;
 
 	for (count = 0; envp[count] != NULL; count++) ;
@@ -61,7 +64,7 @@ char **env_copy(char **envp) {
 		for (i = 0; i < count; i++)
 			if ((p[i] = strdup(envp[i])) == NULL) {
 				save_errno = errno;
-				while (--i >= 0)
+				while (i-- > 0)
 					free(p[i]);
 				free(p);
 				errno = save_errno;
@@ -73,21 +76,21 @@ char **env_copy(char **envp) {
 }
 
 char **env_set(char **envp, const char *envstr) {
-	int count, found;
+	size_t count, found;
 	char **p, *envtmp;
 
 	/*
 	 * count the number of elements, including the null pointer;
 	 * also set 'found' to -1 or index of entry if already in here.
 	 */
-	found = -1;
+	found = (size_t)-1;
 	for (count = 0; envp[count] != NULL; count++) {
 		if (!strcmp_until(envp[count], envstr, '='))
 			found = count;
 	}
 	count++;	/* for the NULL */
 
-	if (found != -1) {
+	if (found != (size_t)-1) {
 		/*
 		 * it exists already, so just free the existing setting,
 		 * save our new one there, and return the existing array.
@@ -107,7 +110,7 @@ char **env_set(char **envp, const char *envstr) {
 	if ((envtmp = strdup(envstr)) == NULL)
 		return (NULL);
 	p = (char **) realloc((void *) envp,
-		(size_t) ((count + 1) * sizeof (char *)));
+		(count + 1) * sizeof (char *));
 	if (p == NULL) {
 		free(envtmp);
 		return (NULL);
@@ -135,6 +138,7 @@ int env_set_from_environ(char ***envpp) {
 		"LC_ALL",
 		"LANGUAGE",
 		"RANDOM_DELAY",
+		"MAILFROM",
 		NULL
 	};
 	const char **name;
@@ -179,20 +183,16 @@ int load_env(char *envstr, FILE * f) {
 	long filepos;
 	int fileline;
 	enum env_state state;
-	char name[MAX_ENVSTR], val[MAX_ENVSTR];
-	char quotechar, *c, *str;
+	char quotechar, *c, *str, *val;
 
 	filepos = ftell(f);
 	fileline = LineNumber;
-	skip_comments(f);
 	if (EOF == get_string(envstr, MAX_ENVSTR, f, "\n"))
 		return (ERR);
 
 	Debug(DPARS, ("load_env, read <%s>\n", envstr));
 
-		memset(name, 0, sizeof name);
-	memset(val, 0, sizeof val);
-	str = name;
+	val = str = envstr;
 	state = NAMEI;
 	quotechar = '\0';
 	c = envstr;
@@ -236,8 +236,9 @@ int load_env(char *envstr, FILE * f) {
 		case EQ1:
 			if (*c == '=') {
 				state++;
-				str = val;
 				quotechar = '\0';
+				*str++ = *c;
+				val = str;
 			}
 			else {
 				if (!isspace((unsigned char) *c))
@@ -258,40 +259,48 @@ int load_env(char *envstr, FILE * f) {
 			abort();
 		}
 	}
-	if (state != FINI && !(state == VALUE && !quotechar)) {
+	if (state != FINI && state != EQ2 && !(state == VALUE && !quotechar)) {
 		Debug(DPARS, ("load_env, not an env var, state = %d\n", state));
-			fseek(f, filepos, 0);
+		if (fseek(f, filepos, 0)) {
+			return ERR;
+		}
 		Set_LineNum(fileline);
 		return (FALSE);
 	}
+	*str = '\0';
 	if (state == VALUE) {
 		/* End of unquoted value: trim trailing whitespace */
-		c = val + strlen(val);
-		while (c > val && isspace((unsigned char) c[-1]))
-			*(--c) = '\0';
+		while (str > val && isspace((unsigned char)str[-1]))
+			*(--str) = '\0';
 	}
-
-	/* 2 fields from parser; looks like an env setting */
-
-	/*
-	 * This can't overflow because get_string() limited the size of the
-	 * name and val fields.  Still, it doesn't hurt to be careful...
-	 */
-	if (!glue_strings(envstr, MAX_ENVSTR, name, val, '='))
-		return (FALSE);
-	Debug(DPARS, ("load_env, <%s> <%s> -> <%s>\n", name, val, envstr));
-		return (TRUE);
+	return TRUE;
 }
 
 char *env_get(const char *name, char **envp) {
-	int len = strlen(name);
+	size_t len = strlen(name);
 	char *p, *q;
 
 	while ((p = *envp++) != NULL) {
 		if (!(q = strchr(p, '=')))
 			continue;
-		if ((q - p) == len && !strncmp(p, name, len))
+		if ((size_t)(q - p) == len && !strncmp(p, name, len))
 			return (q + 1);
 	}
 	return (NULL);
 }
+
+char **env_update_home(char **envp, const char *dir) {
+	char envstr[MAX_ENVSTR];
+
+	if (dir == NULL || *dir == '\0' || env_get("HOME", envp)) {
+		return envp;
+	}
+
+	if (glue_strings(envstr, sizeof envstr, "HOME", dir, '=')) {
+		envp = env_set(envp, envstr);
+	}			
+	else
+		log_it("CRON", getpid(), "ERROR", "can't set HOME", 0);
+
+	return envp;
+}
