@@ -49,7 +49,6 @@ void		set_cron_uid(void),
 		unget_char(int, FILE *),
 		free_entry(entry *),
 		acquire_daemonlock(int),
-		skip_comments(FILE *),
 		log_it(const char *, PID_T, const char *, const char *, int),
 		log_close(void),
 		check_orphans(cron_db *);
@@ -68,17 +67,19 @@ int		load_database(cron_db *),
 		swap_uids_back(void),
 		load_env(char *, FILE *),
 		env_set_from_environ(char ***envpp),
+		cron_pabort(FILE *),
 		cron_pclose(FILE *),
 		glue_strings(char *, size_t, const char *, const char *, char),
 		strcmp_until(const char *, const char *, char),
-		allowed(const char * ,const char * ,const char *),
-		strdtb(char *);
+		skip_comments(FILE *),
+		allowed(const char * ,const char * ,const char *);
 
-size_t		strlens(const char *, ...);
+size_t		strlens(const char *, ...),
+		strdtb(char *);
 
 char		*env_get(const char *, char **),
 		*arpadate(time_t *),
-		*mkprints(unsigned char *, unsigned int),
+		*mkprints(unsigned char *, size_t),
 		*first_word(const char *, const char *),
 		**env_init(void),
 		**env_copy(char **),
