@@ -143,7 +143,7 @@ int strcmp_until(const char *left, const char *right, char until) {
 
 /* strdtb(s) - delete trailing blanks in string 's' and return new length
  */
-int strdtb(char *s) {
+size_t strdtb(char *s) {
 	char *x = s;
 
 	/* scan forward to the null
@@ -166,7 +166,7 @@ int strdtb(char *s) {
 	/* the difference between the position of the null character and
 	 * the position of the first character of the string is the length.
 	 */
-	return (x - s);
+	return ((size_t)(x - s));
 }
 
 int set_debug_flags(const char *flags) {
@@ -359,8 +359,8 @@ void acquire_daemonlock(int closeflag) {
 
 	sprintf(buf, "%ld\n", (long) pid);
 	(void) lseek(fd, (off_t) 0, SEEK_SET);
-	len = strlen(buf);
-	if ((num = write(fd, buf, len)) != len)
+	len = (ssize_t)strlen(buf);
+	if ((num = write(fd, buf, (size_t)len)) != len)
 		log_it("CRON", pid, "ERROR", "write() failed", errno);
 	else {
 		if (ftruncate(fd, num) == -1)
@@ -380,7 +380,7 @@ int get_char(FILE * file) {
 	ch = getc(file);
 	if (ch == '\n')
 		Set_LineNum(LineNumber + 1)
-			return (ch);
+	return (ch);
 }
 
 /* unget_char(ch, file) : like ungetc but do LineNumber processing
@@ -415,14 +415,20 @@ int get_string(char *string, int size, FILE * file, const char *terms) {
 
 /* skip_comments(file) : read past comment (if any)
  */
-void skip_comments(FILE * file) {
+int skip_comments(FILE * file) {
 	int ch;
+	int n = 0;
 
 	while (EOF != (ch = get_char(file))) {
 		/* ch is now the first character of a line.
 		 */
-		while (ch == ' ' || ch == '\t')
+		if (++n > MAX_GARBAGE)
+			return FALSE;
+		while (ch == ' ' || ch == '\t') {
 			ch = get_char(file);
+			if (++n > MAX_GARBAGE)
+				return FALSE;
+		}
 
 		if (ch == EOF)
 			break;
@@ -437,85 +443,18 @@ void skip_comments(FILE * file) {
 		 * character on a line.
 		 */
 
-		while (ch != '\n' && ch != EOF)
+		while (ch != '\n' && ch != EOF) {
 			ch = get_char(file);
-
+			if (++n > MAX_GARBAGE)
+				return FALSE;
+		}
 		/* ch is now the newline of a line which we're going to
 		 * ignore.
 		 */
 	}
 	if (ch != EOF)
 		unget_char(ch, file);
-}
-
-/* int in_file(const char *string, FILE *file, int error)
- *	return TRUE if one of the lines in file matches string exactly,
- *	FALSE if no lines match, and error on error.
- */
-static int in_file(const char *string, FILE * file, int error) {
-	char line[MAX_TEMPSTR];
-	char *endp;
-
-	if (fseek(file, 0L, SEEK_SET))
-		return (error);
-	while (fgets(line, MAX_TEMPSTR, file)) {
-		if (line[0] != '\0') {
-			endp = &line[strlen(line) - 1];
-			if (*endp != '\n')
-				return (error);
-			*endp = '\0';
-			if (0 == strcmp(line, string))
-				return (TRUE);
-		}
-	}
-	if (ferror(file))
-		return (error);
-	return (FALSE);
-}
-
-/* int allowed(const char *username, const char *allow_file, const char *deny_file)
- *	returns TRUE if (allow_file exists and user is listed)
- *	or (deny_file exists and user is NOT listed).
- *	root is always allowed.
- */
-int allowed(const char *username, const char *allow_file,
-	const char *deny_file) {
-	FILE *fp;
-	int isallowed;
-	char buf[128];
-
-	if (getuid() == 0)
-		return TRUE;
-	isallowed = FALSE;
-	if ((fp = fopen(allow_file, "r")) != NULL) {
-		isallowed = in_file(username, fp, FALSE);
-		fclose(fp);
-		if ((getuid() == 0) && (!isallowed)) {
-			snprintf(buf, sizeof (buf),
-				"root used -u for user %s not in cron.allow", username);
-			log_it("crontab", getpid(), "warning", buf, 0);
-			isallowed = TRUE;
-		}
-	}
-	else if ((fp = fopen(deny_file, "r")) != NULL) {
-		isallowed = !in_file(username, fp, FALSE);
-		fclose(fp);
-		if ((getuid() == 0) && (!isallowed)) {
-			snprintf(buf, sizeof (buf),
-				"root used -u for user %s in cron.deny", username);
-			log_it("crontab", getpid(), "warning", buf, 0);
-			isallowed = TRUE;
-		}
-	}
-#ifdef WITH_AUDIT
-	if (isallowed == FALSE) {
-		int audit_fd = audit_open();
-		audit_log_user_message(audit_fd, AUDIT_USER_START, "cron deny",
-			NULL, NULL, NULL, 0);
-		close(audit_fd);
-	}
-#endif
-	return (isallowed);
+	return TRUE;
 }
 
 void log_it(const char *username, PID_T xpid, const char *event,
@@ -648,7 +587,7 @@ char *first_word(const char *s, const char *t) {
 /* warning:
  *	heavily ascii-dependent.
  */
-static void mkprint(char *dst, unsigned char *src, int len) {
+static void mkprint(char *dst, unsigned char *src, size_t len) {
 /*
  * XXX
  * We know this routine can't overflow the dst buffer because mkprints()
@@ -659,10 +598,10 @@ static void mkprint(char *dst, unsigned char *src, int len) {
 
 		if (ch < ' ') {	/* control character */
 			*dst++ = '^';
-			*dst++ = ch + '@';
+			*dst++ = (char)(ch + '@');
 		}
 		else if (ch < 0177) {	/* printable */
-			*dst++ = ch;
+			*dst++ = (char)ch;
 		}
 		else if (ch == 0177) {	/* delete/rubout */
 			*dst++ = '^';
@@ -679,7 +618,7 @@ static void mkprint(char *dst, unsigned char *src, int len) {
 /* warning:
  *	returns a pointer to malloc'd storage, you must call free yourself.
  */
-char *mkprints(unsigned char *src, unsigned int len) {
+char *mkprints(unsigned char *src, size_t len) {
 	char *dst = malloc(len * 4 + 1);
 
 	if (dst)
@@ -712,8 +651,8 @@ char *arpadate(time_t *clock) {
 #endif /*MAIL_DATE */
 
 #ifdef HAVE_SAVED_UIDS
-	static uid_t save_euid;
-	static gid_t save_egid;
+static uid_t save_euid;
+static gid_t save_egid;
 
 int swap_uids(void) {
 	save_egid = getegid();
